function HtmlTreeBuilder(branch, leaf) {
	this.branch = branch;
	this.leaf = leaf;
	this.idCount = 0;
	
	// External function - draws the SVG node as a tree, item should be an svgTree.js Item object
	function drawTree(item) {
		var html = getSvgTagOpen(item);
		var children = item.getChildren();
		for(var i = 0; i < children.length; i++) {
			var br ='';
			br =  this.addBranch(children[i], br, 1);
			html += br;
		}
		html += '</li> \n </ul> \n';
		return html;
	};
	
	function addBranch(item, html, level) {
		var children = item.getChildren();
		if (children.length > 0) {
			html += this.extractInfo(item, true, level);
			for(var i = 0; i < children.length; i++) {
				var branch = '';
				 branch = this.addBranch(children[i], branch, level + 1);
				 html += branch;
			}
			html += getCloseTag(this.branch) +'\n';
		} else {
			html = this.addLeaf(item, html, level);
		}
		return html;
	};

	function addLeaf(item, html, level) {
		var leaf = this.extractInfo(item, false, level);
		return leaf + getCloseTag(this.leaf) +'\n';
	};
	
	function extractInfo(item, isBranch, level, cssClass) {
		var tag = item.getTag();
		var id2=this.getId();
		var id = (isBranch) ? ' id=' + id2 : '';
		var leaf = this.getOpenTag(isBranch, id, level, cssClass);
		var label = 'tag: ' + tag;
		
		var role = item.getRole();
		if (role) {
			label += ' role: ' + role;
		}
		
		if (tag == 'title' || tag == 'desc' || tag == 'text') {
			label += ' text: ' + item._content;
		}
		
		if (isBranch) {
			var li = this.extractInfo(item, false, level, 'tree-parent');
			return li + leaf;
		}
		return leaf + label;
	};
	
	
	function getOpenTag(isBranch, id, level, cssClass) {	
		var tag = (isBranch) ? this.branch : this.leaf;
		var role = '';
		
		if (isBranch) {
			role = ' role="group"';
		} else {
			role = ' role ="treeitem"';
		}
		
		var tabIndex = ((level > 2 && isBranch) || level > 3) ? ' tabindex="-1"' : ' tabindex="0"';	
		var cClass = (cssClass) ? ' class="' + cssClass + '"' : '';
		if (!isBranch && cssClass) {
			cClass += (level > 3) ? ' aria-expanded="false"' : ' aria-expanded="true"';
		}
		
		return '<' + tag + id + role + tabIndex + cClass + '>';
		
	};
	
	function  getCloseTag(tag) {
		return '</' + tag + '>';
	};
	
	function getId() {
		return '"b' + this.idCount++ + '"';
	}
	
	function getSvgTagOpen(item) {
		var label = '<h2 id="svgRootLabel">svg</h2> \n';
		var svgTag = '<ul  class="tree root-level" role="tree" aria-labeledby="svgRootLabel" id="svgTreeUl"> \n';
		return label + svgTag;
	}
	
	this.addBranch = addBranch;
	this.addLeaf = addLeaf;
	this.getId = getId;
	this.getOpenTag = getOpenTag;
	this.extractInfo = extractInfo;
	this.drawTree = drawTree;
};